#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"
# Tcl/Tk Benchmark Program for MS Windows
#
# created by Keiichi Takahashi, 10/13/96
#                       Reviced 11/07/97
# bench.tcl
#
proc version {} {
	global w
	set font1 "{Times New Roman} 16 {}"
	set font2 "{Arial} 12 {}"
	toplevel .hlp
	wm title .hlp "Version"
	frame .hlp.msg1 -relief flat -borderwidth 2 -bg white
	label .hlp.msg1.title1 -text "Tcl/Tk Benchmark Ver.1.3" -font $font1 -background white
	label .hlp.msg1.title2 -text "Keiichi Takahashi, 11/7/97" -font $font2 -background white
	label .hlp.msg1.title3 -text "ktaka@geocities.com" -font $font2 -background white
	label .hlp.msg1.title4 -image tcllogo -background white

	pack .hlp.msg1.title1 .hlp.msg1.title2 .hlp.msg1.title3
	pack .hlp.msg1.title4
	pack .hlp.msg1 -fill both

	frame .hlp.info -bg white
	set c 3
	set m1 ".hlp.info.list1"
	set m2 ".hlp.info.list2"

	listbox $m1 -bg white -height $c -width 10 -font $font2 -borderwidth 0
	$m1 insert end "Tcl/Tk"
	$m1 insert end "Machine"
	$m1 insert end "OS"

	listbox $m2 -bg white -height $c -font $font2 -borderwidth 0
	$m2 insert end $w(tcltk)
	$m2 insert end $w(machine)
	$m2 insert end $w(os)

	pack $m1 $m2 -side left -expand yes

	pack .hlp.info -fill both
	
	frame .hlp.ok -bg white
	button .hlp.ok.b -text " OK " -font $font2 -command { destroy .hlp }
	pack .hlp.ok.b -expand yes
	pack .hlp.ok -fill both
	focus .hlp
}
# SAVE
proc SaveDone {} {
	set font1 "{Arial} 12 {}"
	if { ![ catch { toplevel .done }]} {
		wm title .done "Completed Save Data"
		label .done.msg -text " The result has been saved in result.txt." -font $font1 -borderwidth 10
		button .done.bttn -text " OK " -font $font1 -command { destroy .done }
		pack .done.msg
		pack .done.bttn
	}
	focus .done
}
proc SaveResults {} {
	global result0 result1 result2 result3 w
	set fileid [open "result.txt" "w"]

	set titlename "Tcl/Tk Benchmark Results\n"
	puts $fileid $titlename

	set headname "Tcl/Tk:  $w(tcltk)"
	puts $fileid $headname
	set headname "Machine: $w(machine)"
	puts $fileid $headname
	set headname "OS:      $w(os)"
	puts $fileid $headname

	set txtname {{ LINE : } { PAINT: } { MOVE : } { TOTAL: }}
	set uname { {sec } {%  } {} }
	lappend result $result0 $result1 $result2 $result3

	puts $fileid ""
	puts $fileid " ITEM      TIME     RATIO      TIMESTAMP"
	puts $fileid "--------------------------------------------------"
	for { set i 0 } { $i < 4 } { incr i } {
		set data [ lindex $txtname $i ]
		set datan [ lindex $result $i ]
		for { set j 0 } { $j < 3 } { incr j } {
			append data [ lindex $datan $j ] [ lindex $uname $j ]
		}
		puts $fileid $data
	}
	close $fileid
	SaveDone
}
# TIMESTAMP
proc GetTimeStamp { } {
	set now_time [ clock seconds ]
	set timestamp [ clock format $now_time -format "%b-%d-%y %T" ]
	return $timestamp
}
# FORMAT
# usage
# n: Variable to Format
# a: Number of Characters including Decimal point after formating
# b: Number of Characters of Decimal Place after formating
proc FormatNumber { n a b } {
	set l [ string length $n ]
	set p 0
	set f "."
	set z "0"
	set s " "
	while { ([ string index $n $p ] != "." )&&( $p < $l ) } { incr p }
	if { $p == $l } then {
		set n $n$f
		incr l
		for { set i 0 } { $i < $b } { incr i } {
			set n $n$z
			incr l
		}
	}
	while { [ expr $l-$p-1 ] < $b } {
		set n $n$z
		incr l
	}
	while { [ expr $l-$p-1 ] > $b } {
		set n [ string range $n 0 [ expr $l-2 ]]
		incr l -1
	}
	while { $l < $a } {
		set n $s$n
		incr l
	}
	return $n
}
# RANDOM
proc randomInit { seed } {
	global rand
	set rand(ia) 9301
	set rand(ic) 49297
	set rand(im) 233280
	set rand(seed) $seed
}
proc random {} {
	global rand
	set rand(seed) [ expr ($rand(seed)*$rand(ia)+$rand(ic))%$rand(im)]
	return [ expr $rand(seed)/double($rand(im))]
}
proc randomRange { range } {
	expr int([random]*$range)
}
# Benchmark 1
proc DrawLine {} {
	global basetime1
	if { ![ catch { toplevel .win }]} {
		wm title .win "Drawing Area"
		canvas .win.draw -background white -width 400 -height 300
		pack .win.draw
	}
	focus .win
	.win.draw delete tags all
	set lcolor { black gray cyan blue green yellow orange magenta red }
	set ttime [ time {
		for { set i 1 } { $i <= 400 } { incr i } {
			.win.draw create line $i 1 $i 300 \
				-fill [ lindex $lcolor [ expr int( fmod( $i, 9 ))]] -tags v
			update
		}
		.win.draw delete tags v
		for { set i 1 } { $i <= 300 } { incr i } {
			.win.draw create line 1 $i 400 $i \
				-fill [ lindex $lcolor [ expr int( fmod( $i, 9 ))]] -tags h
			update
		}
		.win.draw delete tags h
	} 3 ]
	set basetime $basetime1
	set ratio [ FormatNumber [ expr $basetime/[ lindex $ttime 0 ]*100. ] 5 1 ]
	set ttime [ FormatNumber [ expr [ lindex $ttime 0 ]/1000000.] 7 2 ]
	set rtime [ GetTimeStamp ]
	focus .
	return [ list $ttime $ratio $rtime ]
}
# Benchmark 2
proc PaintOval {} {
	global basetime2
	if { ![ catch { toplevel .win }]} {
		wm title .win "Drawing Area"
		canvas .win.draw -background white -width 400 -height 300
		pack .win.draw
	}
	focus .win
	.win.draw delete tags all
	randomInit [ clock seconds ]
	set fcolor { black gray cyan blue green yellow orange magenta red }
	set ttime [ time {
		for { set i 0 } { $i < 100 } { incr i } {
			.win.draw create oval [ randomRange 400 ] [ randomRange 300 ] \
				[ randomRange 400 ] [ randomRange 300 ] \
				-fill [ lindex $fcolor [ randomRange 9 ] ] -tags o
			update
		}
		.win.draw delete tags o
	} 3 ]
	set basetime $basetime2
	set ratio [ FormatNumber [ expr $basetime/[ lindex $ttime 0 ]*100. ] 5 1 ]
	set ttime [ FormatNumber [ expr [ lindex $ttime 0 ]/1000000.] 7 2 ]
	set rtime [ GetTimeStamp ]
	focus .
	return [ list $ttime $ratio $rtime ]
}
# Benchmark 3
proc MoveBall {} {
	global basetime3
	if { ![ catch { toplevel .win }]} {
		wm title .win "Drawing Area"
		canvas .win.draw -background white -width 400 -height 300
		pack .win.draw
	}
	focus .win
	.win.draw delete tags all
	
	set ttime [ time {
		set y 15
		set pitch 8.5
		while { $y < 280 } {
			for { set x 10 } { $x < 390 } { incr x } {
				.win.draw delete tags m
				.win.draw create image $x [ expr $y+$x/$pitch ] -image ball -tags m
				update
			}
			set y [ expr $y+$x/$pitch ]
			bell
			for { set x 390 } { $x > 10 } { incr x -1 } {
				.win.draw delete tags m
				.win.draw create image $x [ expr $y+(390-$x)/$pitch ] -image ball -tags m
				update
			}
			set y [ expr $y+(390-$x)/$pitch ]
			bell
		}
	.win.draw delete tags m
	} 3 ]
	set basetime $basetime3
	set ratio [ FormatNumber [ expr $basetime/[ lindex $ttime 0 ]*100. ] 5 1 ]
	set ttime [ FormatNumber [ expr [ lindex $ttime 0 ]/1000000.] 7 2 ]
	set rtime [ GetTimeStamp ]
	focus .
	return [ list $ttime $ratio $rtime ]
}
# Menu
proc MakeMenu {} {
	set font1 "{Arial} 12 {}"
	frame .menubar -relief raised -borderwidth 2
	# File
	menubutton .menubar.file -text "File" -font $font1 -underline 0 -menu .menubar.file.menu
	menu .menubar.file.menu -tearoff no
	.menubar.file.menu add command -label "Exec" -font $font1 -underline 0 -command ExecAll
	.menubar.file.menu add command -label "Save" -font $font1 -underline 0 -command SaveResults
	.menubar.file.menu add separator
	.menubar.file.menu add command -label "Exit" -font $font1 -underline 1 -command exit
	menubutton .menubar.help -text "Help" -font $font1 -underline 0 -menu .menubar.help.menu
	# Help
	menu .menubar.help.menu -tearoff no
	.menubar.help.menu add command -label "Version" -font $font1 -underline 0 -command version
	# PACK
	pack .menubar.file -side left
	pack .menubar.help -side right
	pack .menubar -side top -fill x -expand true
}
# BUTTON
proc MakeButtonMenu {} {
	frame .menubttn -relief ridge -borderwidth 2
	button .menubttn.b1 -image disk -command SaveResults
	button .menubttn.b2 -image help -command version
	button .menubttn.b3 -image pexit -command exit
	pack .menubttn.b1 -side left
	pack .menubttn.b3 .menubttn.b2 -side right
	pack .menubttn -side top -fill x -expand true
}
# Main Panel
proc DispResult { w result } {
	$w.t1 configure -text [ lindex $result 0 ]
	$w.t2 configure -text [ lindex $result 1 ]
	$w.d1 configure -text [ lindex $result 2 ]
}
proc ExecLine {} {
	global result0
	set result0 [ DrawLine ]
	DispResult ".panel0" $result0
}
proc ExecPaint {} {
	global result1
	set result1 [ PaintOval ]
	DispResult ".panel1" $result1
}
proc ExecMove {} {
	global result2
	set result2 [ MoveBall ]
	DispResult ".panel2" $result2
}
proc ExecAll {} {
	global result0 result1 result2 result3
	ExecLine
	ExecPaint
	ExecMove
	set ttime [ FormatNumber [ expr ([ lindex $result0 0 ]+[ lindex $result1 0 ]+[ lindex $result2 0 ])] 7 2]
	set ratio [ FormatNumber [ expr ([ lindex $result0 1 ]+[ lindex $result1 1 ]+[ lindex $result2 1 ])/3. ] 5 1]
	set rtime [ GetTimeStamp ]
	set result3 [ list $ttime $ratio $rtime ]
	DispResult ".panel3" $result3
}
proc MakeMainPanel {} {
	set font1 "{Courier New} 12 {}"
	set w ".panel"
	set imgname { marker palett snoopy wheel }
	set cmdname { ExecLine ExecPaint ExecMove ExecAll }
	set txtname {{LINE  } {PAINT } {MOVE  } {TOTAL }}
	for { set i 0 } { $i < 4 } { incr i } {
		set wn $w$i
		frame $wn -relief flat -borderwidth 2
		label $wn.s1 -text " "
		button $wn.b1 -image [ lindex $imgname $i ] -command [ lindex $cmdname $i ]
		label $wn.l1 -text [ lindex $txtname $i ] -font $font1
		label $wn.t1 -text "????.??" -font $font1 -foreground green -background black
		label $wn.u1 -text "sec " -font $font1
		label $wn.t2 -text "???.?" -font $font1 -foreground green -background black
		label $wn.u2 -text "% " -font $font1
		label $wn.d1 -text "???-??-?? ??:??:??" -font $font1 -foreground green -background black
		label $wn.s2 -text " "
		pack $wn.s1 -side left
		pack $wn.b1 -side left
		pack $wn.l1 $wn.t1 $wn.u1 $wn.t2 $wn.u2 $wn.d1 $wn.s2 -side left
		pack $wn  -side top -fill x -expand true
	}
}
# ##########
#    MAIN
# ##########

global wwm title . "Tcl/Tk Benchmark"
bell

image create photo disk    -file ico/disk.gif
image create photo help    -file ico/help.gif
image create photo pexit   -file ico/exit.gif
image create photo marker  -file ico/marker.gif
image create photo palett  -file ico/palett.gif
image create photo snoopy  -file ico/snoopy.gif
image create photo wheel   -file ico/wheel.gif
image create photo tcllogo -file ico/TclTk.gif
image create photo ball    -file ico/ball.gif
set result0 {{} {} {}}
set result1 {{} {} {}}
set result2 {{} {} {}}
set result3 {{} {} {}}
set basetime1 2500000.
set basetime2 3300000.
set basetime3 7000000.
# set tcl_precision 12

set w(tcltk)	"$tcl_patchLevel / $tk_patchLevel"
set w(machine)	$tcl_platform(machine)
set w(os)	"$tcl_platform(os) $tcl_platform(osVersion)"

MakeMenu
MakeButtonMenu
MakeMainPanel
# END
